/*	Sample program for Olimex AVR-MT with AT90S2313 processor
 *	Echoes back the received characters on the UART.
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#define __AVR_AT90S2313__	1
#define OSCSPEED	10000000

#include "avr/io.h"

void PORT_Init()
{
	PORTB = 0b00000000;
	DDRB  = 0b00010000;			//set Buzzer as output (Bit4 = 1)

	PORTD = 0b00000000;
	DDRD  = 0b00001010;			//set Relay as output (Bit3 = 1), set TX as output(Bit1 = 1)
}

void UART_Init(uint32_t Baud)
{
	//Calculate BaudRate (Datasheet page 44)
	unsigned char BaudRate = OSCSPEED / (16 * Baud) - 1;

	//set BaudRate into register
	UBRR = BaudRate;

	UCR = UCR | 0b00011000;		//enable TX(Bit3 = 1) and RX(Bit4 = 1)
}

unsigned char UART_Receive()
{
	if (USR & 0b10000000)		//if there is unreaded data
		return UDR;
	else
		return 0;				//no unreaded data
}

void UART_Transmit(unsigned char Data)
{
	while (!(USR & 0b00100000));//waiting until buffer is ready to receive
	UDR = Data;
}

int main()
{
	PORT_Init();
	UART_Init(9600);
	unsigned char Ch;
	while (1)
	{
		Ch = UART_Receive();
		if (Ch) UART_Transmit(Ch);
	}
}
